#ifndef PAPI_INT_H_INCLUDED
#define PAPI_INT_H_INCLUDED
/**********************************************************************
 *
 * Open Standard Print API (PAPI) - Internal Functions
 *
 * Copyright (c) 2002 International Business Machines Corp.
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 *********************************************************************/
#include "papi.h"

/* Service handle */
typedef struct
{
	/* Standard data... */
	char* 	service_name;
	char*	user_name;
	char*	password;
	int (*authCB)(papi_service_t svc);
	papi_attribute_t**	status;
	papi_attribute_t**	capabilities;
	papi_encryption_t	encryption;
	void*	app_data;
	/* Implementation-specific data... */
	char    full_service_name[HTTP_MAX_URI];
	char	hostname[HTTP_MAX_URI];
	char	object_uri[IPP_MAX_NAME];
	char	language[IPP_MAX_NAME];	/* attributes-natural-language */
	char	char_set[IPP_MAX_NAME];	/* attributes-character-set */
	int     port;
	http_t*	http;
	ipp_t*  request;
	ipp_t*  response;
	papi_status_t last_error;
} papi_int_service_t;

/* Job object */
typedef struct
{
    char* name;                    /* Printer name or URI */
    int32_t id;                    /* Local job ID number */
    papi_attribute_t** attributes; /* Job attributes      */
    papi_job_ticket_t* job_ticket; /* Job ticket          */
} papi_int_job_t;

/* Stream object */
typedef struct
{
	char	printer[IPP_MAX_NAME];
	char	temp_filename[IPP_MAX_NAME];
	FILE*   tempf;
} papi_int_stream_t;

/* Printer object */
typedef struct
{
    char* name;                    /* Printer name or URI */
    papi_attribute_t** attributes; /* Printer attributes  */
} papi_int_printer_t;

/* To use a opaque API handes as pointers to the real structures */
#define INT_HANDLE(h) ((papi_int_service_t*)h)
#define INT_JOB_HANDLE(h) ((papi_int_job_t*)h)
#define INT_PRTR_HANDLE(h) ((papi_int_printer_t*)h)
#define INT_STREAM(h) ((papi_int_stream_t*)h)

/* PAPI_RETURN: Return from top-level PAPI function to calling code */
#define PAPI_RETURN(f, h, rc) \
{ \
	if (h) \
	{ \
		if (rc != PAPI_OK) \
			((papi_int_service_t*)h)->last_error = rc; \
		if ( ((papi_int_service_t*)h)->http ) \
		{ \
			httpClose( ((papi_int_service_t*)h)->http ); \
			((papi_int_service_t*)h)->http = NULL; \
		} \
	} \
	if (f) \
		papi_dbg_printf("<%s returning 0x%x at line %d\n", f, rc, __LINE__); \
	return(rc); \
}

/* Type of object */
typedef enum
{
	PAPI_PRINTER,
	PAPI_JOB
} papi_object_type_t;

/* Platform-specific directory delimiter */
#ifdef _WINDOWS
#define DIR_DELIM_CH '\\'
#define DIR_DELIM_STR "\\"
#else
#define DIR_DELIM_CH '/'
#define DIR_DELIM_STR "/"
#endif


/* Function Declarations */
extern void papi_init();
extern papi_status_t papi_set_up_request(papi_int_service_t* handle,
		const char* name, const papi_object_type_t type);
extern void papi_get_service_info(papi_int_service_t* handle,
		const char* name, const papi_object_type_t type);
extern papi_status_t papi_validate_and_connect(papi_int_service_t* handle,
		const char* name, const papi_object_type_t type);
extern void papi_clear_prev_request(papi_int_service_t* handle);
extern int papi_get_array_count(void** array);
extern void papi_add_common_args(papi_int_service_t* handle);
extern void papi_do_request(papi_int_service_t* handle, const char* resource,
							const char** files);
extern papi_status_t papi_get_result(papi_int_service_t* handle);

extern papi_status_t papi_attr_cups2papi(const ipp_attribute_t* cups_attrs,
                           papi_attribute_t*** papi_attrs);
extern papi_status_t papi_attr_papi2cups(const papi_attribute_t** papi_attrs,
						   ipp_t* request, const char* char_set, const ipp_tag_t attr_group);
extern void papi_values_free(papi_attribute_t* attr);
extern void papi_attr_free(papi_attribute_t* attr);
extern ipp_attribute_t* papi_add_cups_attr(ipp_t* req, const int num_values);
extern ipp_t* papi_dup_cups_request(ipp_t* req);

extern void papi_job_ticket_cups2jt(ipp_attribute_t* jt_attr, papi_job_ticket_t* jt);

extern void papi_dbg_printf(const char* fmt, ...);
extern void papi_dbg_list_cups_attribute(ipp_attribute_t* attr);
extern void papi_dbg_list_all_cups_attributes(ipp_t* response, const char* str);
extern void papi_dbg_list_some_cups_attributes(ipp_t* response, const char* attrs[],
	int num_attrs, const char* str);

#endif
